@@ -1,3 +1,4 @@
+import collections
 import io
 import logging
 import os
@@ -8,8 +9,10 @@ import ruamel.yaml
 import xapian
 import jinja2
 
-from pkg_resources import resource_string
-from . import types
+from pkg_resources import resource_string, resource_filename
+from . import types, templating
+
+logger = logging.getLogger(__name__)
 
 try:
     import git
@@ -56,7 +59,7 @@ class Entry(object):
             else:
                 data[name] = field.empty()
             if field.field.get('help', None) is not None:
-                comment = "# {name} ({type}): {help}\n".format(**field.field)
+                comment = "{name} ({type}): {help}\n".format(**field.field)
                 try:
                     data.yaml_set_comment_before_after_key(
                         key=name,
@@ -64,7 +67,24 @@ class Entry(object):
                         indent=0
                     )
                 except AttributeError:
-                    logging.warning(
+                    logger.warning(
+                        "Not adding comments because they are not"
+                        "supported by the yaml loader."
+                    )
+            valid_values = field.field.get('values', [])
+            if valid_values:
+                comment = "{name} ({type}): {valid_values}".format(
+                    valid_values="|".join(valid_values),
+                    **field.field
+                )
+                try:
+                    data.yaml_set_comment_before_after_key(
+                        key=name,
+                        before=comment,
+                        indent=0
+                    )
+                except AttributeError:
+                    logger.warning(
                         "Not adding comments because they are not"
                         "supported by the yaml loader."
                     )
@@ -160,7 +180,7 @@ class Collection(object):
             try:
                 self.stemmer = xapian.Stem(self.settings['lang'])
             except xapian.InvalidArgumentError:
-                logging.warning(
+                logger.warning(
                     "Invalid language %s, in settings.yaml: using english.",
                     self.settings['lang'],
                 )
@@ -194,7 +214,7 @@ class Collection(object):
                 # unknown fields are treated as if they were
                 # (unvalidated) generic YAML to support working with
                 # collections based on lesana derivatives
-                logging.warning(
+                logger.warning(
                     "Unknown field type %s in field %s",
                     field['type'],
                     field['name'],
@@ -208,7 +228,7 @@ class Collection(object):
         entry = self.entry_class(self, data, fname)
         valid, errors = entry.validate()
         if not valid:
-            logging.warning(
+            logger.warning(
                 "Not indexing {fname}: invalid data".format(fname=fname)
             )
             return False, errors
@@ -267,7 +287,7 @@ class Collection(object):
             try:
                 fnames = os.listdir(self.itemdir)
             except FileNotFoundError:
-                logging.warning(
+                logger.warning(
                     "No such file or directory: {}, not updating cache".format(
                         self.itemdir
                     )
@@ -278,14 +298,14 @@ class Collection(object):
             try:
                 valid, errors = self._index_file(fname, cache)
             except IOError as e:
-                logging.warning(
+                logger.warning(
                     "Could not load file {}: {}".format(fname, str(e))
                 )
             else:
                 if valid:
                     updated += 1
                 else:
-                    logging.warning(
+                    logger.warning(
                         "File {fname} could not be indexed: {errors}".format(
                             fname=fname, errors=errors
                         )
@@ -300,18 +320,18 @@ class Collection(object):
 
     def git_add_files(self, files=[]):
         if not git_available:
-            logging.warning(
+            logger.warning(
                 "python3-git not available, could not initalise "
                 + "the git repository."  # noqa: W503
             )
             return False
         if not self.settings.get('git', False):
-            logging.info("This collection is configured not to use git")
+            logger.info("This collection is configured not to use git")
             return False
         try:
             repo = git.Repo(self.basedir, search_parent_directories=True)
         except git.exc.InvalidGitRepositoryError:
-            logging.warning(
+            logger.warning(
                 "Could not find a git repository in {}".format(self.basedir)
             )
             return False
@@ -324,13 +344,20 @@ class Collection(object):
                 os.path.join(self.basedir, '.lesana/xapian'),
             )
         except xapian.DatabaseOpeningError:
-            logging.info("No database found, indexing entries.")
+            logger.info("No database found, indexing entries.")
             self.update_cache()
             cache = xapian.Database(
                 os.path.join(self.basedir, '.lesana/xapian'),
             )
         return cache
 
+    def render_query_template(self, query):
+        """
+        Render a query template, filling it with search_aliases.
+        """
+        t = jinja2.Template(query)
+        return t.render(**self.settings.get('search_aliases', {}))
+
     def start_search(self, querystring, sort_by=None):
         """
         Prepare a search for querystring.
@@ -402,6 +429,48 @@ class Collection(object):
             doc = cache.get_document(post.docid)
             yield self._doc_to_entry(doc)
 
+    def get_field_values(self, field, querystring='*'):
+        field = self.fields[field]
+        if field.field.get('sortable', False):
+            self.start_search(querystring)
+
+            spy = xapian.ValueCountMatchSpy(field.value_index)
+            self._enquire.add_matchspy(spy)
+
+            cache = self._get_cache()
+            self._enquire.get_mset(0, cache.get_doccount())
+
+            for v in spy.values():
+                yield {
+                    'value': v.term,
+                    'frequency': v.termfreq,
+                }
+
+        else:
+            logger.info(
+                "Trying to get the list of values for a non sortable field."
+            )
+            logger.info(
+                "This is going to be pretty inefficient."
+            )
+            if field.field['type'] == 'list':
+                values = []
+                for e in self.get_all_documents():
+                    values.extend(e.data[field.field['name']])
+            else:
+                values = (
+                    e.data[field.field['name']]
+                    for e in self.get_all_documents()
+                )
+            logger.info("Values are %s", str(values))
+
+            counter = collections.Counter(values)
+            for v in counter.most_common():
+                yield {
+                    'value': v[0],
+                    'frequency': v[1],
+                }
+
     def _match_to_entry(self, match):
         return self._doc_to_entry(match.document)
 
@@ -441,7 +510,7 @@ class Collection(object):
                     cache.delete_document(entry.idterm)
                     self.remove_file(entry.fname)
                 else:
-                    logging.warning("No such entry: {}, ignoring".format(eid))
+                    logger.warning("No such entry: {}, ignoring".format(eid))
         cache.commit()
         cache.close()
 
@@ -451,7 +520,7 @@ class Collection(object):
             try:
                 repo = git.Repo(self.basedir, search_parent_directories=True)
             except git.exc.InvalidGitRepositoryError:
-                logging.warning(
+                logger.warning(
                     "Could not find a git repository in {}".format(
                         self.basedir
                     )
@@ -473,7 +542,7 @@ class Collection(object):
         self.update_cache([e.fname for e in changed])
 
     def get_template(self, template_fname, searchpath='.'):
-        env = jinja2.Environment(
+        env = templating.Environment(
             loader=jinja2.FileSystemLoader(
                 searchpath=searchpath, followlinks=True,
             ),
@@ -482,9 +551,29 @@ class Collection(object):
         try:
             template = env.get_template(template_fname)
         except jinja2.exceptions.TemplateNotFound as e:
-            raise TemplatingError('Could not find template' + str(e))
+            raise TemplatingError('Could not find template ' + str(e))
         return template
 
+    def entry_from_rendered_template(self, template, data):
+        try:
+            template = self.get_template(template)
+            rendered = template.render(**data)
+        except jinja2.exceptions.TemplateSyntaxError as e:
+            raise TemplatingError(e)
+        try:
+            data = ruamel.yaml.load(rendered, ruamel.yaml.RoundTripLoader)
+        except ruamel.yaml.YAMLError as e:
+            logger.warning(
+                "The following data failed to load as YAML: \n{}".format(
+                    rendered
+                )
+
+            )
+            raise TemplatingError(e)
+        entry = self.entry_class(self, data=data)
+        self.save_entries([entry])
+        return entry
+
     @classmethod
     def init(
         cls, directory=None, git_enabled=True, edit_file=None, settings={}
@@ -506,7 +595,7 @@ class Collection(object):
             if git_available:
                 repo = git.Repo.init(c_dir, bare=False)
             else:
-                logging.warning(
+                logger.warning(
                     "python3-git not available, could not initalise "
                     + "the git repository."  # noqa: W503
                 )
@@ -527,7 +616,20 @@ class Collection(object):
                     fp.write('#Added by lesana init\n.lesana')
                 if repo:
                     repo.index.add(['.gitignore'])
-            # TODO: Add hook to index files as they are pulled
+
+            # Add post-checkout and post-merge hook
+            hook_source = resource_filename('lesana', 'data/post-checkout')
+            hooks_dir = os.path.join(
+                c_dir,
+                '.git',
+                'hooks',
+            )
+            checkout_hook = os.path.join(hooks_dir, 'post-checkout')
+            merge_hook = os.path.join(hooks_dir, 'post-merge')
+
+            shutil.copy(hook_source, checkout_hook)
+            if not os.path.islink(merge_hook):
+                os.symlink(checkout_hook, merge_hook)
         # If it doesn't exist, create a skeleton of settings.yaml file
         # then open settings.yaml for editing
         filepath = os.path.join(c_dir, 'settings.yaml')
