@@ -12,6 +12,8 @@ import dateutil.parser
 
 import xapian
 
+logger = logging.getLogger(__name__)
+
 
 class LesanaType:
     """
@@ -37,6 +39,25 @@ class LesanaType:
         """
         return value
 
+    def allowed_value(self, value):
+        """
+        Check whether a value is allowed in this field.
+
+        Return the value itself or raise LesanaValueError if the value
+        isn't valid.
+        """
+        if not value:
+            return value
+        valid_values = self.field.get('values')
+        if not valid_values:
+            return value
+        if value in valid_values:
+            return value
+        raise LesanaValueError("Value {} is not allowed in field {}".format(
+            value,
+            self.field.get('name')
+        ))
+
     def _to_index_text(self, value):
         """
         Prepare a value for indexing.
@@ -62,7 +83,7 @@ class LesanaType:
         if not to_index:
             return
         if not value:
-            logging.info(
+            logger.info(
                 "Not indexing empty value {}".format(value)
             )
             return
@@ -75,8 +96,8 @@ class LesanaType:
             if self.value_index and self.value_index >= 16:
                 doc.add_value(self.value_index, self._to_value(value))
             else:
-                logging.debug(
-                    "Index values up to 8 are reserved for internal use"
+                logger.debug(
+                    "Index values up to 15 are reserved for internal use"
                 )
 
 
@@ -89,7 +110,7 @@ class LesanaString(LesanaType):
     def load(self, data):
         if not data:
             return data
-        return str(data)
+        return self.allowed_value(str(data))
 
     def empty(self):
         return ""
@@ -112,7 +133,7 @@ class LesanaInt(LesanaType):
         if not data:
             return data
         try:
-            return int(data)
+            return self.allowed_value(int(data))
         except ValueError:
             raise LesanaValueError(
                 "Invalid value for integer field: {}".format(data)
@@ -147,7 +168,7 @@ class LesanaInt(LesanaType):
             if int(increment) == increment:
                 return value + increment
             else:
-                logging.warning(
+                logger.warning(
                     "Invalid configuration value for increment in field %s: "
                     + "%s",
                     self.field['name'],
@@ -167,7 +188,7 @@ class LesanaFloat(LesanaType):
         if not data:
             return data
         try:
-            return float(data)
+            return self.allowed_value(float(data))
         except ValueError:
             raise LesanaValueError(
                 "Invalid value for float field: {}".format(data)
@@ -179,7 +200,13 @@ class LesanaFloat(LesanaType):
 
 class LesanaDecimal(LesanaType):
     """
-    A floating point number
+    A fixed point number
+
+    Because of a limitation of the yaml format, these should be stored
+    quoted as a string, to avoid being loaded back as floats.
+
+    Alternatively, the property ``precision`` can be used to force all
+    values to be rounded to that number of decimals.
     """
     name = "decimal"
 
@@ -187,11 +214,15 @@ class LesanaDecimal(LesanaType):
         if not data:
             return data
         try:
-            return decimal.Decimal(data)
+            value = decimal.Decimal(data)
         except decimal.InvalidOperation:
             raise LesanaValueError(
                 "Invalid value for decimal field: {}".format(data)
             )
+        precision = self.field.get('precision')
+        if precision:
+            value = round(value, precision)
+        return self.allowed_value(value)
 
     def empty(self):
         return decimal.Decimal(0)
@@ -278,7 +309,7 @@ class LesanaDate(LesanaType):
         # compatibility with dateutil before 2.8
         ParserError = getattr(dateutil.parser, 'ParserError', ValueError)
         try:
-            return dateutil.parser.parse(data)
+            return dateutil.parser.parse(data).date()
         except ParserError:
             raise LesanaValueError(
                 "Invalid value for date field: {}".format(data)
@@ -341,6 +372,21 @@ class LesanaURL(LesanaString):
     name = 'url'
 
 
+class LesanaGeo(LesanaString):
+    """
+    A Geo URI
+    """
+    name = 'geo'
+
+    def load(self, data):
+        data = super().load(data)
+        if data and not data.startswith("geo:"):
+            raise LesanaValueError("{} does not look like a geo URI".format(
+                data
+            ))
+        return data
+
+
 class LesanaYAML(LesanaType):
     """
     Free YAML contents (no structure is enforced)
@@ -366,7 +412,7 @@ class LesanaList(LesanaType):
         try:
             self.sub_type = types[field['list']](field, types)
         except KeyError:
-            logging.warning(
+            logger.warning(
                 "Unknown field type %s in field %s",
                 field['type'],
                 field['name'],
@@ -378,7 +424,7 @@ class LesanaList(LesanaType):
             # empty for this type means an empty list
             return []
         try:
-            return [self.sub_type.load(x) for x in data]
+            return [self.allowed_value(self.sub_type.load(x)) for x in data]
         except TypeError:
             raise LesanaValueError(
                 "Invalid value for list field: {}".format(data)
